const RootTypes = `
    # Requêtes
    type Query {
        accessGroups : GroupQuery
        accessPosts : MessageQuery
        accessUsers : UserQuery

        asAdmin(groupUID: ID): AdminQuery
        asSpeaker(groupUID: ID): AdminQuery
        asMember(groupUID: ID): AdminQuery
        asViewer(groupUID: ID): AdminQuery
        
    }

    type Mutation {
        asAdmin(groupUID: String): AdminMutation
        asSpeaker(groupUID: String): SpeakerMutation
        asMember(groupUID: String): MemberMutation
        asViewer(groupUID: String): ViewerMutation
    }

   
`;

const subMutations = `
    type AdminMutation {

        isAdmin: Boolean

        createSubgroup(
            uid: ID = null,
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

        addUser(userid : String): User
        removeUser(userid : String): User
        addAdmin(userid : String): User
        removeAdmin(userid : String): User

        editGroup(
            name: String,
            website: String,
            description: String,
            school: String
        ): Group

    }

    type SpeakerMutation{
        postEvent(name: String, date: String): Event
        answerRequest(request: ID, accept : Boolean): Request
    }

    type MemberMutation {
        leave: Group
    }

    type ViewerMutation {
        requestJoin: Group
    }


`;

const subQueries = `

    """
    Requête pour obtenir un groupe.
    """
    type GroupQuery{
        allGroups: [Group]
        group(uid: ID) : Group
        simpleGroup(uid : ID) : SimpleGroup
        metaGroup(uid : ID) : MetaGroup

    }

    """
    Requête pour obtenir un message.
    """
    type MessageQuery{
        allMessages: [Message]
        allEvents: [Event]
        allPosts: [Post]
        post(id: ID): Post
        allAnnouncements: [Announcement]
    }

    type UserQuery{

        user(uid: ID) : User

        searchTOL(
            givenName: String,
            lastName: String,
            nickname: String,
            nationality: String,
            school: String,
            promotion: String,
            groups: String,
            studies: String,
            sport: String,
            phone: String,
            mail: String,
            adress: String,
            ip: String
        ): [String]
    }

    # Requête à la base de donnée nécessitant d'être administrateur.
    type AdminQuery{
        isAdmin: Boolean
        allRequests : AllRequests
    }

    type SpeakerQuery{
        isSpeaker: Boolean
    }

    type MemberQuery{
        isMember: Boolean
    }

    type ViewerQuery{
        isViewer: Boolean
    }
`;

const User = `
    # Utilisateurs
    type User {
        # Prénom de l'utilisateur
        givenName: String!
        # Nom de famille
        lastName: String!
        uid: ID!
        birthdate: String!
        mail: String
        phone: String
        # Groupes dont l'utilisateur est membre.
        groups: [SimpleGroup]
        # Groupes que l'utilisateur aime.
        likes: [Group]
        address: String
    }
`;

const Group = `
    # Groupes associatifs

    """
    L'interface Group représente les deux types de groupes implémentés dans Sigma : les groupes
    simples, dont les membres sont des utilisateurs, et les métagroupes, dont les membres sont
    des groupes simples (tel que Federez, dont les membres incluent le BR et DaTA). 
    """
    interface Group {
        # Identifiant unique de l'union
        uid: ID
        # Nom de l'union
        name: String
        # Site du groupe
        website: String
        # Le groupe est-il un spiritualGroup (sinon c'est un materialGroup)
        createdAt: String!
        updatedAt: String!

    }

    # Le groupe de base, dont les membres sont des utilisateurs. Binets, Kes, etc...
    type SimpleGroup implements Group{
        uid: ID
        name: String
        website: String
        createdAt: String!
        updatedAt: String!

        # Membres du groupe
        members: [User]
        # Jour et heure de création du groupe
        description: String
        # École d'origine du groupe
        school: String
        # Groupe parent
        parent: Group
    }

    # Anciennement appelé "interface dans les discussions"
    # Un groupe dont les membre sont d'autres groupes
    type MetaGroup implements Group {
        uid: ID
        name: String
        website: String
        createdAt: String!
        updatedAt: String!

        # Les groupes constitutifs du méta-groupe.
        members: [Group]!
    }

`;

const Message = `
    # Tout type de message créé par un ou plusieurs groupes.
    interface Message {
        id: ID!
        # Titre du message
        title: String!
        createdAt: String!
        updatedAt: String!
        # Auteur(s) du message
        authors: [Group]
    }
`;

const Post = `
    # Publication postée par un ou plusieurs groupes.
    type Post implements Message {
        id: ID!
        title: String!
        createdAt: String!
        updatedAt: String!
        authors: [Group]
        content: String!
    }
`;

const Announcement = `
    # Annonce publique effectuée par un ou plusieurs groupes.
    type Announcement implements Message {
        id: ID!
        title: String!
        createdAt: String!
        updatedAt: String!
        authors: [Group]
        description: String!
        views: Int
    }
`;

const Event = `
    # Événements organisés par un ou plusieurs groupes.
    type Event implements Message {
        id: ID!
        # Intitulé de l'événement
        title: String!
        # Lieu de l'événement
        location: String
        createdAt: String!
        updatedAt: String!
        startTime: String!
        endTime: String!
        # Organisateurs
        authors: [Group]
        # Personnes qui participent à l'événement.
        participants: [User]
        description: String
    }
`;

const Requests = `
    # Demandes effectuées au groupe.
    type AllRequests {
        userJoinGroup: [UserJoinGroup]
        groupJoinEvent: [GroupJoinEvent]
        yourGroupHostEvent: [YourGroupHostEvent]
    }

    interface Request {
        # ID de la demande
        id: ID!
        # message accompagnant la demande
        message: String
    }

    # Demande d'un utilisateur désirant rejoindre le groupe.
    type UserJoinGroup implements Request{
        id: ID!
        message: String
        # Émetteur de la demande
        user: User
    }


    # Demande d'un groupe voulant rejoindre un événement
    type GroupJoinEvent implements Request{
        id: ID!
        message: String
        # Événement concerné
        event: Event
        # Groupe voulant rejoindre l'événement
        groupWantingToJoin: Group
    }

    # Demande au récipiendaire de rejoindre l'organisation d'un événement.
    type YourGroupHostEvent implements Request{
        id: ID!
        message: String
        # Événement concerné
        event: Event
        # Groupe ayant publié l'évènement et lancé l'invitation
        sender: Group
    }

`;

const typeDefs = [RootTypes, Group, User, Message, Post, Announcement, Event, subMutations, subQueries, Requests];
export default typeDefs;
