/**
 * @file Ce fichier gère les requêtes LDAP de type données ; liste des groupe d'un individu, liste des membres d'un groupe... A ne pas confondre avec ldap_auth qui lui gère l'authentification. Il est à lire conjointement avec config.json qui détaille les paxs et champs spécifique à un LDAP. Ce fichier permet en outre de changer facilement les paramètres d'une fonction et de mieux comprendre les formats attendus en entrée. Il est fortement conseillé d'utiliser [`JXplorer`](http://jxplorer.org/) pour naviguer sur le LDAP en parallèle du dévelopement. Toutes les fonctions écrites ici sont asynchrones et renvoient des Promises ce qui nécessite de les appeler avec la synthaxe un peu particulière `f(args).then(res => ...)` pour exploiter leur résultat.
 * @author hawkspar
 */

/* Ne pas hésiter à repasser en synthaxe ES5... Plus simple pour tester en solo avec node directement
import ldap  from 'ldapjs';
import fs from 'fs';
import ldapEscape from 'ldap-escape';
import path from 'path'; */

/**/var ldap = require('ldapjs');
var fs = require('fs');
var ldapEscape = require('ldap-escape');
var path = require('path');

// Important ; permet de vérifier que l'utilisateur reste connecté.
var ensureLoggedin =  require('connect-ensure-login').ensureLoggedIn;

// Point central ; tous les champs de la BDD sont 'cachés' dans config.json et pas visibles directement
var configPath = path.resolve('./','ldap_config.json');
var config = JSON.parse(fs.readFileSync(configPath, 'utf8'));
// Idem pour les ids de connexion temporaires
var connectPath = path.resolve('./','ldap_connexion_config.json');
var connect = JSON.parse(fs.readFileSync(connectPath, 'utf8'));

// Connection au serveur LDAP avec des temps de timeout arbitraires
var client = ldap.createClient({ url: config.ldap.server});

//------------------------------------------------------------------------------------------------------------------------
// Fonctions de base
//------------------------------------------------------------------------------------------------------------------------

/**
 * @summary Fonction qui sert à s'identifier sur le LDAP. Ne renvoie rien.
 * @desc Assez important en terme de sécurité, de gestion de conflit, et de droit d'accès. Méthode ldapjs 
 * (voir [`Client API`](http://ldapjs.org/client.html) méthode bind).
 * @arg {Object} user - Utilisateur de la forme suivante :
 * @arg {string} user[uid] - User identifier
 * @arg {string} user[password] - Mot de passe
*/
function connecterLDAP(user) {
    // TBM utiliser user
    client.bind(connect.dn, connect.passwd, (err, res) => {});
}

/**
 * @summary Fonction qui interroge le LDAP selon un protocole spécifié en argument et renvoit les valeurs trouvées.
 * @desc Cette fonction appelle {@link bind} pour authentifier l'utilisateur pour authentifier l'utilisateur puis rompts la connexion. Cette fonction utilise ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode search). Il faut l'appeler suivant un schéma `rechercherLDAP(user, ...).then((res) => { truc avec res });`. Cette fonction fait une demande au LDAP qu'elle filtre selon un schéma prédéfini dans `dic` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} base - DN de l'emplacement de la requête
 * @arg {string} filter ["(objectClass=*)"] - Filtre logique de la recherche (format [`RFC2254`](https://tools.ietf.org/search/rfc2254)) déjà passé au ldapEscape
 * @arg {string} filter_dic[key] - Vraie valeur pertinente de la recherche
 * @arg {string[]} attributes - Liste des attributs qui figureront dans le résultat final
 * @return {Promise(Object[])} Résultats de la recherche ; soit une liste de valeurs d'attributs, 
 * soit une liste de dictionnaires si on veut plus d'un attribut (les clés du dictionnaire sont celles du LDAP) 
 */
function rechercherLDAP(user, base, attributes, filter="(objectClass=*)") {
    return new Promise(function(resolve, reject) {
        connecterLDAP(user);
        //Debug
        /*console.log(base);
        console.log(filter);
        console.log(attributes);*/

        let vals=[];
        // Interrogation LDAP selon configuration fournie en argument
        client.search(base, {
            "scope": "sub",
            "filter": filter,
            "attributes": attributes
        }, function(err, res) {
            // Gestion erreur
            if (err) {
                reject(err);
            } else {
                // Dès que la recherche renvoit une entrée, on stocke les attributs qui nous intéresse
                res.on('searchEntry', function(entry) {
                    // Cas un seul attribut où le résultat est une liste directement
                    if (!Array.isArray(attributes)) { vals.push(entry.object[attributes]); }
                    else if (attributes.length == 1) { vals.push(entry.object[attributes[0]]); }
                    // Cas plusieurs attributs donc résultat dictionnaire
                    else {
                        vals.push({});
                        attributes.forEach((attribute) => {
                            vals.slice(-1)[0][attribute]=entry.object[attribute];
                        });
                    }
                });
                // Si la recherche est finie on se déconnecte et on renvoit la liste
                res.on('end', function(res) {
                    client.bind("", "", (err, res) => {});
                    resolve(vals);
                });
            }
        });
    });
}

/**
 * @summary Fonction qui permet de modifier un élément sur le LDAP. Gestion intelligente de l'appartenance à un binet.
 * @desc Cette fonction appelle {@link bind} pour authentifier l'utilisateur pour authentifier l'utilisateur puis rompts la connexion. Cette fonction utilise une Promise pour être asynchrone ; elle renvoit la promesse d'une réponse puis traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode modify). Il faut l'appeler suivant un schéma `modifierLDAP(...).then((res) => { truc avec res });`. Cette fonction fait une demande au LDAP qu'elle filtre selon un schéma prédéfini dans `dic` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} name - DN de l'endroit à modifier
 * @arg {string} op - Operation à réaliser sur le LDAP. Trois opération sont possibles ; "add", qui rajoute des attributs et qui peut créer des doublons, "del" qui en supprime, et "replace" qui remplace du contenu par un autre. 
 * @arg {Object} mod - Dictionnaire contenant les attributs à modifier et les nouvelles valeurs des attributs. Une nouvelle valeur vide ("") est équivalent à la suppression de cet attribut.
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function modifierLDAP(user, name, op, mod) {
    return new Promise(function(resolve, reject) {
        connecterLDAP(user);
        // Modification LDAP selon configuration en argument (pourrait prendre une liste de Changes)
        client.modify(name, new ldap.Change({
            operation: op,
            modification: mod,
        }), function(err) {
            reject(err);
        });
        client.bind("", "", (err, res) => {});
        resolve(true);
    });
}

/**
 * @summary Fonction qui permet de rajouter un élément sur le LDAP.
 * @desc Cette fonction appelle {@link connecterLDAP} pour authentifier l'utilisateur puis rompts la connexion. Cette fonction utilise une Promise pour être asynchrone ; elle renvoit la promesse d'une réponse puis traite la demande avec ldapjs (voir [`Client API`](http://ldapjs.org/client.html) méthode add). Il faut l'appeler suivant un schéma `modifierLDAP(...).then((res) => { truc avec res });`. Cette fonction fait une demande au LDAP qu'elle filtre selon un schéma prédéfini dans `dic` et à chaque résultat (event SearchEntry) le met dans une liste, et renvoit la liste à l'issue (event end).
 * @arg {Object} user - Utilisateur de la forme nécessaire au {@link connecterLDAP}
 * @arg {string} dn - Adresse du parent
 * @arg {Object} vals - Dictionnaire contenant les valeurs à créer
 * @arg {string} vals[key] - Nouvelle valeur pour le champ key
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function ajouterLDAP(user, dn, vals) {
    return new Promise(function(resolve, reject) {
        connecterLDAP(user);
        // Ajout LDAP selon la configuration en argument
        client.add(config.key_id+"="+vals["uid"]+","+dn, vals, function(err) {
            reject(err);
        });
        client.bind("", "", (err, res) => {});
        resolve(true);
    });
}

//------------------------------------------------------------------------------------------------------------------------
// Fonctions de lecture
//------------------------------------------------------------------------------------------------------------------------

/**
 * @summary Fonction qui retrouve les groupes dont un individu est membre.
 * @desc Cette fonction utilise {@link rechercherLDAP} avec un dictionnaire prédéfini dans config.json, entrée lg. Elle va directement à la bonne feuille et n'a donc pas de filtre. Elle utilise ldapEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant de l'individu à interroger (le plus souvent prenom.nom, parfois l'année, supposé valide)
 * @return {Promise(string[])} Liste des uid de groupes (noms flat des groupes) où l'id fourni est membre
 */
function listerGroupes(user, uid) {
    return new Promise(function(resolve, reject) {
        rechercherLDAP(user,ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id : uid}), config.lg.attributs).then(res => resolve(res[0]));
    });
}

/**
 * @summary Fonction qui retrouve la liste des membres d'un groupe.
 * @desc Cette fonction utilise {@link rechercherLDAP} avec un dictionnaire prédéfini dans config.json. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} gid - Identifiant du groupe à interroger (le plus souvent nom du groupe en minuscule)
 * @return {Promise(String[])} Liste des uid des membres où l'id fournie est membre (noms flat des groupes)
 */
function listerMembres(user, gid) {
    return new Promise(function(resolve, reject) {
        rechercherLDAP(user,ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id : gid}), config.lm.attributs).then(res => resolve(res[0]));
    });
}

/**
 * @summary Fonction qui retrouve la liste des admins d'un groupe.
 * @desc Cette fonction utilise {@link rechercherLDAP} avec un dictionnaire prédéfini dans config.json. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} gid - Identifiant du groupe à interroger (le plus souvent nom du groupe en minuscule)
 * @return {Promise(string[])} Liste des uid des membres où l'id fournie est membre (noms flat des groupes)
 */
function listerAdministrateurs(user, gid) {
    return new Promise(function(resolve, reject) {
        rechercherLDAP(user,ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id : gid}), config.la.attributs).then(res => resolve(res[0]));
    });
}
 
/**
 * @summary Fonction qui renvoit toutes les infos relatives à un hruid particulier.
 * @desc Cette fonction utilise {@link rechercherLDAP} avec un dictionnaire prédéfini dans config.json. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant de l'utilisateur
 * @return {Promise(Object[])} Informations recueillies ; renvoie une liste de dictionnaire avec les mêmes clés que dans le TOL, voir config.json.
 */
function renseignerSurUtilisateur(user, uid) {
    return new Promise(function(resolve, reject) {
        rechercherLDAP(user,ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id : uid}), config.rs.attributs).then(res => resolve(res));
    });
}

//------------------------------------------------------------------------------------------------------------------------
// Fonctions de recherche
//------------------------------------------------------------------------------------------------------------------------

/**
 * @summary Fonction qui interroge le LDAP et retrouve les groupes (voir LDAP) qui ressemblent
 *  à l'entrée. Etape 0 vers un vrai TOL (Trombino On Line).
 * @desc Cette fonction utilise {@link rechercherLDAP} mais avec un filtre généré à la volée. Accepte des champs exacts ou incomplets mais pas approximatifs et ne gère pas l'auto-complete. Cette fonction utilise aussi config.json. MEF Timeout pour des recherches trop vagues. Renvoit une liste d'uid. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} input - String entré par l'utilisateur qui ressemble au nom du groupe.
 * @return {Promise(string[])} Liste des uid de groupes dont le nom ressemble à l'input
 */
function trouverGroupes(user, input) {
    return new Promise(function(resolve, reject) {
        // Escape de l'input utilisateur par sécurité
        let str=ldapEscape.filter("${txt}", { txt: input});

        // Construction du filtre custom
        let filter= "(|("+config.key_id+"="+str+")"+      // On cherche la valeur exacte
                    "(|("+config.key_id+"=*"+str+")"+     // La valeur finale avec des trucs avant ; wildcard *
                    "(|("+config.key_id+"=*"+str+"*)"+    // La valeur du milieu avec des trucs avant et après
                    "("+config.key_id+"="+str+"*))))";   // La valeur du début avec des trucs après

        // Appel rechercheLDAP avec filtre de l'espace 
        rechercherLDAP(user, config.dn_groups, config.key_id, filter).then(res => resolve(res));
    });
}

// ASKIP elle crash
/**
 * @summary Fonction qui interroge le LDAP et retrouve le groupe qui ressemblent à l'input et qui correspond. Etape 0 vers un vrai TOL (Trombino On Line).
 * @desc Cette fonction utilise {@link rechercherLDAP} mais avec un filtre généré à la volée. Accepte des champs exacts ou incomplets mais pas approximatifs et ne gère pas l'auto-complete. Cette fonction utilise aussi config.json. MEF Timeout pour des recherches trop vagues. Renvoit une liste d'uid. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} input - String entré par l'utilisateur qui ressemble au nom du groupe.
 * @arg {string} type - String aux valeurs prédéfinies dans ldap_config.
 * @return {Promise(string[])} Liste des uid de groupes dont le nom ressemble à l'input
*/
function trouverGroupesParTypes(user, input, type) {
    return new Promise(function(resolve, reject) {
        // Appels imbriqués
        trouverGroupes(user, input).then(gidList => {
            let gidtyList = [];
            gidList.forEach(gid => {
                // Nouvelle recherche à partir des résultats de la première
                rechercherLDAP(ldapEscape.filter(config.key_id+"=${id},"+config.dn_groups, {id : gid} ), config.key_id, ldapEscape.filter(config.tgty.filter, { ty: type })).then(res => {
                    if (res.length!=0) { gidtyList.push(res); }
                });
            });
            resolve(gidtyList);
        });
    });
}

/**
 * @summary Fonction qui renvoit les attributs demandés des paxs validant les critères de recherche. Première étape vers vrai TOL (Trombino On Line).
 * @desc Cette fonction utilise {@link rechercherLDAP} mais avec un filtre généré à la volée. Accepte des champs exacts ou incomplets pour la plupart des champs mais pas approximatifs et ne gère pas l'auto-complete. MEF Timeout pour des recherches trop vagues. Elle utilise LDAPEscape pour éviter les injections.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire contenant les données nécessaires à la recherche. Les valeurs sont celles entrées par l'utilisateur et sont par hypothèse comme des sous-parties compactes des valeurs renvoyées. Tous les champs ci-dessous peuvent être indifféremment des listes (par exempl pour chercher un membre de plusieurs groupes) ou des éléments isolés.
 * @arg {string} data[givenName] - Prénom 
 * @arg {string} data[lastName] - Nom
 * @arg {string} data[nickname] - Surnom
 * @arg {string} data[nationality] - Nationalité (non implémentée pour l'instant, pas de format spécifique)
 * @arg {string} data[promotion] - String de l'année de promo
 * @arg {string} data[phone] - String du numéro de portable
 * @arg {string} data[mail] - Adresse mail
 * @arg {string} data[ip] - Une ou des adresses ip
 * @arg {string} data[school] - Ecole d'appartenance (pour l'instant instable)
 * @arg {string} data[groups] - Un ou plusieurs groupes (pas de différence entre membre simple et admin)
 * @arg {string} data[studies] - PA ou autre
 * @arg {string} data[sport] - Section sportive ou autre
 * @arg {string[]} return_values - Liste d'attributs à renvoyer dans le résultat final
 * @return {Promise(Object[])} Liste de dictionnaires de profils en cohérence avec l'input avec pour clés les attributs des profils.
 */
function repliquerTOLModulable(user, data, return_values) {
    return new Promise(function(resolve, reject) {
        let filter="";
        // Iteration pour chaque champ, alourdissement du filtre selon des trucs prédéfini dans config.json encore
        config.tolm.input_names.forEach(name => {
            if ((data[name]!= undefined) & (data[name] != '')) {                    // Si il y a qque chose à chercher pour ce filtre
                if (!Array.isArray(data[name])) { data[name]=[data[name]]; }    // Gestion d'une liste de valeurs à rechercher
                // Iteration pour chaque valeur fournie par l'utilisateur
                data[name].forEach(val => {
                    // Escape de l'input utilisateur
                    let str=ldapEscape.filter("${input}", { input: val});
                    // Traduction en language LDAP
                    let attribute = config.tolm.correspondance[name];
                    // Creation du filtre étape par étape
                    filter="(&"+filter+ "(|("+attribute+"="+str+")"+      // On cherche la valeur exacte
                                        "(|("+attribute+"=*"+str+")"+     // La valeur finale avec des trucs avant ; wildcard * (MEF la wildcart ne marche pas pour tous les attributs)
                                        "(|("+attribute+"=*"+str+"*)"+    // La valeur du milieu avec des trucs avant et après
                                        "("+attribute+"="+str+"*)))))";   // La valeur du début avec des trucs après
                });
            }
        });
        // Appel rechercheLDAP avec filtre de l'espace 
        rechercherLDAP(user, config.dn_users, return_values, filter).then(res => resolve(res));
    });
}

/**
 * @summary Fonction qui retrouve les uid des paxs validant les critères de recherche. Autre étape vers vrai TOL (Trombino On Line). Doit être préféré à repliquerTOL car moins gourmande envers le LDAP (utiliser {@link renseignerSurUtilisateur} au cas par cas après pour obtenir les vraies infos).
 * @desc Cette fonction utilise {@link repliquerTOLModulable}.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire contenant les données nécessaires à {@link repliquerTOLModulable}
 * @return {Promise(string[])} uids des profils qui "match" les critères proposés.
 */
function repliquerTOLdesIds(user, data) {
    return new Promise( function(resolve, reject) {
        repliquerTOLModulable(user, data, config.key_id).then(res => resolve(res));
    });
}

/**
 * @summary Fonction qui retrouve les paxs validant les critères de recherche. Bien mais vite inutilisable car demande trop au LDAP et renvoie des erreurs de type size limit. Préférer {@link repliquerTOLdesIds} puis {@link renseignerSurUtilisateur} au cas par cas.
 * @desc Cette fonction utilise {@link repliquerTOLModulable} avec un filtre généré à la volée. Accepte des champs incomplets mais pas approximatifs et ne gère pas l'auto-complete.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire contenant les données nécessaires à {@link repliquerTOLModulable}
 * @return {Promise(Object[])} Liste de dictionnaires de profils en cohérence avec l'input avec pour clés tous les attributs disponibles ou presque (voir config).
 */
function repliquerTOL(user, data) {
    return new Promise( function(resolve, reject) {
        repliquerTOLModulable(user, data, config.tol.attributes).then(res => resolve(res));
    });
}

//------------------------------------------------------------------------------------------------------------------------
// Fonctions de création
//------------------------------------------------------------------------------------------------------------------------

/**
 * @summary Fonction qui créé un nouvel utilisateur dans le LDAP.
 * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ; voir ldap_config.json pour le détail de la génération des champs finaux. Appelle {@link ajouterLDAP} bien sûr, mais aussi {@link ajouterMembreGroupe} et {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire des informations utilisateurs
 * @arg {string} data[hruid] - prenom.nom ou nom.promo
 * @arg {string} data[givenName] - Prénom
 * @arg {string} data[lastName] - Nom
 * @arg {string} data[nickname] [] - Surnom
 * @arg {string} data[birthdate] - Date d'anniversaire au format annee-mois-jour
 * @arg {string} data[uid] - Numéro d'identifiant plus ou moins arbitraire
 * @arg {string} data[password] - Mot de passe
 * @arg {string} data[promo] - Simple année d'entrée école (pas d'X17)
 * @arg {string} data[onPlatal] - "String booléen" valant 0 ou 1
 * @arg {string} data[email] - Courriel supposé valide
 * @arg {string} data[phone] - String du numéro de portable
 * @arg {string} data[photo] - Photo jpeg directement en bytestring
 * @arg {string} data[room] - Pour l'instant juste le numéro de casert ou rien, à terme l'adresse complète
 * @arg {string} data[readPerm] [] - Permission spéciales BR sous la forme *truc,
 * @arg {string} data[writePerm] [] - Permission spéciales BR sous la forme *truc,
 * @arg {string[]} data[forlifes] [] - Liste d'alias spéciaux BR (attention le filtre .fkz n'est plus fonctionnel)
 * @arg {string[]} data[ips] - Liste des ips connus de la personne
 * @arg {string[]} data[groupsIsMember] [] - Liste des gid dont le pax est membre
 * @arg {string[]} data[groupsIsAdmin] [] - Liste des gid dont le pax est admin ; supposé sous-liste du précédent
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function creerUtilisateur(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire d'ajout
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cu.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cu.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type ; boucle sur les attributs multiples
            config.cu.multiple_user_infos.forEach(key_att => {
                // Evaluation des expressions des liste de valeurs des attributs
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cu.expr_multiple_values_user[key_att]).forEach(val => {
                    // Nouvel ajout attribut par attribut pour démultiplier les champs
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), "add", { key_att: val });
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data['groupsIsMember'].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data['groupsIsAdmin'].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid);
            });
            resolve(true);
        });
        reject(false);
    });
}

/**
 * @summary Fonction qui créé un nouveau groupe dans le LDAP.
 * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ; c'est mal et cela suppose que beaucoup de soins ont été pris lors de l'escape de ses paramètres. Appelle {@link ajouterLDAP} et {@link modifierLDAP}, mais aussi {@link ajouterMembreGroupe} et {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire des informations utilisateurs (voir détail des champs dans config.json)
 * @arg {string} data[name] - Nom du groupe
 * @arg {string} data[ns] - Statut du groupe ; 'binet' ou 'free', càd ouvert à tous
 * @arg {string} data[gid] - Numéro d'identifiant du groupe
 * @arg {string} data[label] [] - Sorte de surnom ; remplacé par le nom si égal à ''
 * @arg {string[]} data[members] - Liste des membres du groupe
 * @arg {string[]} data[admins] - Liste des admins du groupe ; supposée être une sous-liste de la précédente
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function creerGroupe(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire d'ajout
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cru.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cru.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type
            config.cru.multiple_user_infos.forEach(key_att => {
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cru.expr_multiple_values_user[key_att]).forEach(key_val => {
                    // Nouveau dictionnaire d'ajout
                    let vals2 = { key_att: key_val };
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), "add", vals2);
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data["members"].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data["admins"].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid); });
            resolve(true);
        });
        reject(false);
    });
}

//------------------------------------------------------------------------------------------------------------------------
// Fonctions d'édition
//------------------------------------------------------------------------------------------------------------------------

/**
 * @summary Fonction qui permet de rajouter un membre déjà créé à un groupe.
 * @desc Cette fonction fait essentiellement appel à {@link modifierLDAP} et {@link listerGroupes}. Elle n'autorise pas les doublons et opère dans les deux dns users et groups.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant du futur membre
 * @arg {string} gid - Identifiant du groupe
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function ajouterMembreGroupe(user, uid, gid) {
    return new Promise(function(resolve, reject) {
        // Vérifie que l'utilisateur est pas déjà membre pour groupes
        listerMembres(user,gid).then(lm => {
            if (!lm.includes(uid)) {
                let vals = {};
                vals[config.am.key_gr] = uid;
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_groups, { id: gid }), "add", vals).then(res => {
                    // Erreur si pb lors de la modification
                    if (!res) { reject(false); }
                });
            }
        // Vérifie que l'utilisateur est pas déjà membre pour users
        }).then(res => listerGroupes(user,uid)).then( lg => {
            if (!lg.includes(gid)) {
                let vals2 = {};
                vals2[config.am.key_u] = gid;
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_users, { id: uid }), "add", vals2).then(res => {
                    // Erreur si pb lors de la modification
                    if (!res) { reject(false); }
                });
            }
            resolve(true);
        });
    });
}

/**
 * @summary Fonction qui permet de promouvoir membre au stade d'administrateur d'un groupe.
 * @desc Cette fonction fait essentiellement appel à {@link modifierLDAP}.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant du futur membre
 * @arg {string} gid - Identifiant du groupe
 * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
 */
function ajouterAdministrateurGroupe(user, uid, gid){
    return new Promise(function(resolve, reject) {
        // Vérifie que l'utilisateur est pas déjà membre pour groupes
        listerMembres(user,gid).then(lm => {
            if (!lm.includes(uid)) {
                ajouterMembreGroupe(user,uid,gid).then(res => {
                    // Erreur si pb lors de la modification
                    if (!res) { reject(false); }
                });
            }
        // Vérifie que l'utilisateur est pas déjà membre pour users
        }).then(res => listerGroupes(user,uid)).then( lg => {
            if (!lg.includes(gid)) {
                ajouterMembreGroupe(user,uid,gid).then(res => {
                    // Erreur si pb lors de la modification
                    if (!res) { reject(false); }
                });
            }
        // Attends le résultat des autres puis vérifie que pas déjà admin
        }).then(res => {
            listerAdministrateurs(user,gid).includes(uid).then(la => {
                if (!la.includes(uid)) {
                    // Finalement modification, uniquement dans groups
                    let vals = {};
                    vals[config.aa.key_gr] = uid;
                    modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_groups, { id: gid }), "add", vals).then(res => {
                        // Gestion d'erreur
                        if (!res) { reject(false); }
                    });
                }
            });
            resolve(true);
        });
    });
}

/**
 * @summary Fonction qui permet de supprimer un membre existant d'un groupe.
 * @desc Cette fonction fait essentiellement appel à {@link rechercherLDAP}, {@link modifierLDAP}, {@link listerGroupes} et {@link listerMembres}.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant de l'ex-membre
 * @arg {string} gid - Identifiant du groupe
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function supprimerMembreGroupe(user, uid, gid) {
    return new Promise(function(resolve, reject) {
        // Vérifie que l'utilisateur est pas déjà viré pour groupes
        listerMembres(user,gid).then(lm => {
            if (lm.includes(uid)) {
                // Supprime tous les utilisateurs
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_groups, { id: gid }), "del", config.sm.attributs[0]).then(res => {
                    // Les rajoute un par un, sauf pour le supprimé
                    lm.forEach(id => {
                        if (id!=uid) { ajouterMembreGroupe(user,id, gid); }
                    });
                });
            }
        }).then(res => listerGroupes(user,uid)).then(lg => {
            // Vérifie que l'utilisateur est pas déjà viré pour users
            if (lg.includes(gid)) {
                // Supprime tous les groupes
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_users, { id: uid }), "del", config.sm.attributs[1]).then(res => {
                    // Les rajoute un par un, sauf pour le supprimé
                    lg.forEach(id => {
                        if (id!=gid) { ajouterMembreGroupe(user,uid, id); }
                    });
                });
            }
            resolve(true);
        });
    });
}

//TBC
/**
 * @summary Fonction qui permet de rétrograder un membre du stade d'administrateur d'un groupe au stade d'utilisateur.
 * @desc Cette fonction fait essentiellement appel à {@link modifierLDAP}.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - Identifiant du futur membre
 * @arg {string} gid - Identifiant du groupe
 * @return {boolean} `true` si la modification s'est bien déroulée, false sinon
 */
/*function supprimerAdministrateurGroupe(user, uid, gid) {
    return new Promise(function(resolve, reject) {
        // Vérifie que l'utilisateur est pas déjà viré pour groupes
        listerMembres(user,gid).then(lm => {
            if (lm.includes(uid)) {
                // Supprime tous les utilisateurs
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_groups, { id: gid }), "del", config.sm.attributs[0]).then(res => {
                    // Les rajoute un par un, sauf pour le supprimé
                    lm.forEach(id => {
                        if (id!=uid) { ajouterMembreGroupe(user,id, gid); }
                    });
                });
            }
        }).then(res => listerGroupes(user,uid)).then(lg => {
            // Vérifie que l'utilisateur est pas déjà viré pour users
            if (lg.includes(gid)) {
                // Supprime tous les groupes
                modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_users, { id: uid }), "del", config.sm.attributs[1]).then(res => {
                    // Les rajoute un par un, sauf pour le supprimé
                    lg.forEach(id => {
                        if (id!=gid) { ajouterMembreGroupe(user,uid, id); }
                    });
                });
            }
            resolve(true);
        });
    });
        // Vérifie que l'utilisateur est bien membre pour groupes et le cas échéant le rajoute
        if (!(listerMembres(user,gid).includes(uid) & listerGroupes(user,uid).includes(gid))) {
            ajouterMembreGroupe(user, uid, gid).then(res => {
                // Gestion d'erreur
                if (!res) { reject(false); }
            });
        }
        // Vérifie que l'utilisateur n'est pas déjà admin
        if (!listerAdministrateurs(user,gid).includes(uid)) {
            let vals = {};
            vals[config.am.key_gr] = uid;
            modifierLDAP(user, ldapEscape(config.key_id+"=${id},"+config.dn_groups, { id: gid }), "add", vals).then(res => {
                // Gestion d'erreur
                if (!res) { reject(false); }
            });
            resolve(true);
        }
    });
}*/

//TBC
/**
 * @summary Fonction qui édite un utilisateur existant dans le LDAP. Très similaire à {@link creerUtilisateur}
 * @desc Appelle {@link ajouterLDAP} bien sûr, mais aussi {@link ajouterMembreGroupe} et {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire des informations utilisateurs largement au même format que pour {@link creerUtilisateur} avec tous les champs optionnels
 * @arg {string} data[givenName] [] - Prénom
 * @arg {string} data[lastName] [] - Nom
 * @arg {string} data[nickname] [] - Surnom
 * @arg {string} data[birthdate] [] - Date d'anniversaire au format annee-mois-jour
 * @arg {string} data[password] [] - Mot de passe
 * @arg {string} data[promo] [] - Simple année d'entrée école (pas d'X17)
 * @arg {string} data[onPlatal] [] - "String booléen" valant 0 ou 1
 * @arg {string} data[email] [] - Courriel supposé valide
 * @arg {string} data[phone] [] - String du numéro de portable
 * @arg {string} data[photo] [] - Photo jpeg directement en bytestring
 * @arg {string} data[room] [] - Pour l'instant juste le numéro de casert ou rien, à terme l'adresse complète
 * @arg {string[]} data[ips] [] - Liste des ips connus de la personne
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function editerUtilisateur(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire de modifs
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cu.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cu.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type ; boucle sur les attributs multiples
            config.cu.multiple_user_infos.forEach(key_att => {
                // Evaluation des expressions des liste de valeurs des attributs
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cu.expr_multiple_values_user[key_att]).forEach(val => {
                    // Nouvel ajout attribut par attribut pour démultiplier les champs
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), "add", { key_att: val });
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data['groupsIsMember'].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data['groupsIsAdmin'].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid);
            });
            resolve(true);
        });
        reject(false);
    });
}

//TBC
/**
 * @summary Fonction qui édite un groupe existant dans le LDAP. Très similaire à {@link creerGroupe}
 * @desc Appelle {@link ajouterLDAP} bien sûr, mais aussi {@link ajouterMembreGroupe} et {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {Object} data - Dictionnaire des informations utilisateurs au même format que pour {@link creerGroupe} avec tous les champs optionnels
 * @arg {string} data[name] [] - Nom du groupe
 * @arg {string} data[ns] [] - Statut du groupe ; 'binet' ou 'free', càd ouvert à tous
 * @arg {string} data[label] [] - Sorte de surnom ; remplacé par le nom si égal à ''
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function editerGroupe(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire d'ajout
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cru.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cru.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type
            config.cru.multiple_user_infos.forEach(key_att => {
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cru.expr_multiple_values_user[key_att]).forEach(key_val => {
                    // Nouveau dictionnaire d'ajout
                    let vals2 = { key_att: key_val };
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), "add", vals2);
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data["members"].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data["admins"].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid); });
            resolve(true);
        });
        reject(false);
    });
}

//------------------------------------------------------------------------------------------------------------------------
// Fonctions de suppression
//------------------------------------------------------------------------------------------------------------------------

//TBC
/**
 * @summary Fonction qui supprime un utilisateur du LDAP.
 * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ;
 * voir ldap_config.json pour le détail de la génération des champs finaux. 
 * Appelle {@link ajouterLDAP} bien sûr, mais aussi {@link ajouterMembreGroupe} et {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} uid - uid de la victime
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function supprimerUtilisateur(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire d'ajout
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cu.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cu.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type ; boucle sur les attributs multiples
            config.cu.multiple_user_infos.forEach(key_att => {
                // Evaluation des expressions des liste de valeurs des attributs
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cu.expr_multiple_values_user[key_att]).forEach(val => {
                    // Nouvel ajout attribut par attribut pour démultiplier les champs
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, { id: data["uid"]}), "add", { key_att: val });
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data['groupsIsMember'].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data['groupsIsAdmin'].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid);
            });
            resolve(true);
        });
        reject(false);
    });
}

//TBC
/**
 * @summary Fonction qui supprime un groupe dans le LDAP.
 * @desc Cette fonction fait une utilisation massive d'eval pour anonymiser son code ; c'est mal et cela 
 * suppose que beaucoup de soins ont été pris lors de l'escape de ses paramètres. Appelle {@link ajouterLDAP} 
 * et {@link modifierLDAP}, mais aussi {@link ajouterMembreGroupe} et 
 * {@link ajouterAdministrateurGroupe} pour gérer les groupes du nouvel utilisateur.
 * @arg {Object} user - Utilisateur de la forme nécessaire à {@link connecterLDAP}
 * @arg {string} gid - Identifiant du groupe à supprimer
 * @return {Promise(boolean)} `true` si la modification s'est bien déroulée, false sinon
 */
function supprimerGroupe(user, data) {
    return new Promise(function(resolve, reject) {
        // Calcul d'un dictionnaire d'ajout
        let vals = {};
        // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
        config.cru.single_user_infos.forEach(key_att => vals[key_att]=eval(config.cru.expr_single_values_user[key_att]));
        // Appel à la fonction de base
        ajouterLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), vals).then( res => {
            // Modifications multiples pour avoir plusieurs champs de même type
            config.cru.multiple_user_infos.forEach(key_att => {
                // EVAL DIRECTE D'UN STRING UTILISATEUR ! FAILLE DE SECURITE MAJEURE !
                eval(config.cru.expr_multiple_values_user[key_att]).forEach(key_val => {
                    // Nouveau dictionnaire d'ajout
                    let vals2 = { key_att: key_val };
                    modifierLDAP(user, ldapEscape.filter(config.key_id+"=${id},"+config.dn_users, {id: data['uid']}), "add", vals2);
                });
            });
            // Utilisation des fonctions adaptées pour assurer la cohérence de l'ensemble
            data["members"].forEach(gid => { ajouterMembreGroupe(user, data['uid'], gid); });
            data["admins"].forEach(gid => { ajouterAdministrateurGroupe(user, data['uid'], gid); });
            resolve(true);
        });
        reject(false);
    });
}



//listerGroupes({},"anatole.romon").then(res => console.log(res));
//listerMembres({},"bda").then(res => console.log(res));
//listerAdministrateurs({},"kes").then(res => console.log(res));
//renseignerSurUtilisateur({},"wilson.jallet").then(res => console.log(res));
//trouverGroupes({},"ensta").then(res => console.log(res));
//repliquerTOLdesIds({},{"groups": ["faerix","br"]}).then(res => console.log(res));
//repliquerTOL({},{"groups": ["faerix","br"]}).then(res => console.log(res));
//rechercherLDAP({},"ou=groups,dc=frankiz,dc=net","(cn=f*)",["uid"]).then(res => console.log(res));
//listeAdmins({},"zqfnsflswndf").then(res => console.log(res));


//trouverGroupesParTypes({},"fe","binet").then(res => console.log(res));

/* Partage pour le reste du monde ; même remarque synthaxe que pour l'import
export { listeGroupes, listeMembres, listeAdministrateurs, renseignementsSurUtilisateur, trouverGroupes, repliquerTOLdesIds}; */

/**/module.exports ={ listerGroupes, listerMembres, listerAdministrateurs, renseignerSurUtilisateur, trouverGroupes, repliquerTOLdesIds};
